/*************************************************************************
    fsadm - File system administrator
    1.00 85-09-16 Henrik Norbeck, Luxor Datorer AB
   
    do.c inneh}ller rutinerna som utf|r kommandona
*************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <win/w_macros.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include "fsadm.h"
#include "struct.h"

extern int term;
extern char *clr, *so, *se;
extern int term_h;
extern char fontchar;

extern struct viewport port[];

extern char cwd[];

/*** L{s biblioteket ***/
readdir()
{
    struct direct dirbuf;
    struct stat stbuf;
    register int fd, i;
    char nbuf[PORT_W+1];

    reset(FILES);
    reset(DIRS);
    if ((fd = open(".", O_RDONLY)) == -1)
	message(CANT_READ_DIR, NULL);
    while (read(fd, (char *) &dirbuf, sizeof(dirbuf)) > 0) {
	if (dirbuf.d_ino == 0)
	    continue;
	for (i = 0; i < PORT_W; i++)
	    nbuf[i] = dirbuf.d_name[i];
	nbuf[PORT_W] = '\0';
	if (stat(nbuf, &stbuf) == ERROR)
	    message(NO_STATUS, nbuf);
	insert(S_ISDIR(stbuf.st_mode) ? DIRS : FILES, cwd, nbuf);
	}
    close(fd);
}

/*** G|r en ls -l p} alla filer i en port ***/
do_status(pt)
register int pt;
{
    char *getpath(), *timestr();
    struct stat stbuf;
    register int n = 0, i, j;
    register struct file_info *filep;
    register char *path, *bufp;
    char mode[11], buf[256];

    printf("%s", clr);
    if (term == WINDOW) {
	Rmicons(STDIN);
	refresh(OFF);
	}
    text(STAT_HEADER, NULL);
    for (filep = port[pt].filelist; filep; filep = filep->next) {
	path = getpath(filep);
	if (stat(path, &stbuf) == ERROR) {
	    text(NO_STATUS, path);
	    }
	else {
	    /* Skriv informationen om filen */
	    bufp = buf;
	    /* Slag av fil: */
	    switch (stbuf.st_mode & S_IFMT) {
		case S_IFDIR:
		    mode[0] = 'd';	/* Bibliotek */
		    break;
		case S_IFCHR:
		    mode[0] = 'c';	/* Character special */
		    break;
		case S_IFBLK:
		    mode[0] = 'b';	/* Block special */
		    break;
		default:
		    mode[0] = '-';	/* Vanlig fil */
		    break;
		}
	    /* Filprivilegier: */
	    i = 1; j = 0400;
	    while (i < 10) {
		mode[i] = (stbuf.st_mode & j) ? 'r' : '-';
		j >>= 1; i++;
		mode[i] = (stbuf.st_mode & j) ? 'w' : '-';
		j >>= 1; i++;
		mode[i] = (stbuf.st_mode & j) ? 'x' : '-';
		j >>= 1; i++;
		}
	    mode[10] = '\0';
	    /* Set user id on execution */
	    if (stbuf.st_mode & S_ISUID)
		mode[3] = (mode[3] == 'x') ? 's' : 'S';
	    /* Set group id on execution */
	    if (stbuf.st_mode & S_ISGID)
		mode[6] = (mode[6] == 'x') ? 's' : 'S';
	    /* Save text image */
	    if (stbuf.st_mode & S_ISVTX)
		mode[9] = (mode[9] == 'x') ? 't' : 'T';
	    /* Skriv ut inodnumret, moden och antal l{nkar. */
	    bufp += sprintf(bufp, "%5d %s %2d ",
			stbuf.st_ino, mode, stbuf.st_nlink);
	    /* Skriv {gare: */
	    {
		register struct passwd *pwp;
		struct passwd *getpwuid();

		if ((pwp = getpwuid(stbuf.st_uid)) == NULL)
		    bufp += sprintf(bufp, "%-8d ", stbuf.st_uid);
		else
		    bufp += sprintf(bufp, "%-8s ", pwp->pw_name);
		}
	    /* Skriv grupp: */
	    {
		register struct group *grp;
		struct group *getgrgid();

		if ((grp = getgrgid(stbuf.st_gid)) == NULL)
		    bufp += sprintf(bufp, "%-8d ", stbuf.st_gid);
		else
		    bufp += sprintf(bufp, "%-8s ", grp->gr_name);
	        }
	    /* Skriv storlek eller major & minor nummer: */
	    if (S_ISCHR(stbuf.st_mode))
		bufp += sprintf(bufp, " %3d,%3d ", major(stbuf.st_rdev),
				    minor(stbuf.st_rdev));
	    else 
		bufp += sprintf(bufp, "%8d ", stbuf.st_size);
	    /* Skriv tiden f|r filens skapande, samt filnamnet: */
	    sprintf(bufp, "%s %s", timestr(stbuf.st_mtime), path);
	    printf(buf);
	    }
	putchar('\n');
	if (++n == term_h - 3)
	    if ((n = more(MORE, NULL, n, NULL, 0)) < 0)
		goto exit;
	}
    more(MORE, NULL, 1, NULL, 0);
exit:
    if (term == WINDOW) {
	mkicons();
	refresh(ON);
	}
    draw();
}

/*** Visa filerna i poolen ***/
do_more()
{
    char *getpath();
    struct stat stbuf;
    register int fd, i;
    register struct file_info *filep;
    register char *path;

    printf("%s", clr);
    if (term == WINDOW) {
	Rmicons(STDIN);
	refresh(OFF);
	}
    if (filep = port[POOL].filelist)
	path = getpath(filep);
    while (filep) {
	if ((fd = open(path, O_RDONLY)) == ERROR) {
	    text(UNREADABLE, path);
	    }
	else {
	    if (fstat(fd, &stbuf) == ERROR) {
		text(NO_STATUS, path);
		putchar('\n');
		close(fd);
		}
	    else
		if (S_ISDIR(stbuf.st_mode)) {
		    if ((i = more_dir(fd, path)) == QUIT)
			goto quit;
		    }
		else
		    if ((i = more_file(fd, path)) == QUIT)
			goto quit;
	    putchar('\n');
	    }
next:   filep = filep->next;
	if (filep) {
	    path = getpath(filep);
	    if ((i = more(NEXT_FILE, path, 1, NULL, 0)) == QUIT)
		goto quit;
	    else if (i == NEXT)
		goto next;
	    }
	}
quit:
    if (i >= 0)
	more(MORE, NULL, 1, NULL, 0);
    /* T|m ej poolen efter en more. Man kanske vill g|ra fler saker med
	filerna. */
    if (term == WINDOW) {
	printf("\033(%c\033:J", fontchar);
	mkicons();
	refresh(ON);
	}
    draw();
}

/*** Kopiera filer ***/
do_cp()
{
/*    char name[15];
    char *getpath();
    struct stat stbuf;
    register struct file_info *filep, *nextp;
    register char *path;
    register int dir_flag = FALSE, file_flag = FALSE;

    for (filep = port[POOL].filelist; filep; filep = nextp) {
	nextp = filep->next;
	path = getpath(filep);
	if (stat(path, &stbuf) == ERROR)
	    message(NO_STATUS, path);
	else if (S_ISDIR(stbuf.st_mode))
	    message(ISDIR, path);
	else {
	    ask(LINKNAME, path, name, 15);
	    if (*name == '\0')
		continue;
	    if (*name == '/' OR link(path, name) == ERROR)
		message(CANT_LINK, name);
	    else {
		insert(FILES, cwd, name);
		delete(POOL, filep);
		}
	    }
	}
    pr_port(FILES);
    pr_port(POOL);*/
}

/*** Flytta filer ***/
do_mv()
{
    message(HIT_CE, NULL);
}

/*** Tag bort alla filer i poolen ***/
do_rm()
{
    char *getpath();
    struct stat stbuf;
    register struct file_info *filep, *nextp;
    register char *path;
    register int dir_flag = FALSE, file_flag = FALSE;

    for (filep = port[POOL].filelist; filep; filep = nextp) {
	nextp = filep->next;
	path = getpath(filep);
	if (stat(path, &stbuf) == ERROR)
	    message(NO_STATUS, path);
	else if (S_ISDIR(stbuf.st_mode))
	    if (exec("/bin/rmdir", path) == ERROR)
		message(NO_RMDIR, path);
	    else
		dir_flag |= del_rm(filep, DIRS);
	else
	    if (unlink(path) == ERROR)
		message(NO_RM, path);
	    else
		file_flag |= del_rm(filep, FILES);
	}
    if (file_flag)
	pr_port(FILES);
    if (dir_flag)
	pr_port(DIRS);
    pr_port(POOL);
}

/*** L{nka filer ***/
do_ln()
{
    char name[15];
    char *getpath();
    struct stat stbuf;
    register struct file_info *filep, *nextp;
    register char *path;
    register int dir_flag = FALSE, file_flag = FALSE;

    for (filep = port[POOL].filelist; filep; filep = nextp) {
	nextp = filep->next;
	path = getpath(filep);
	if (stat(path, &stbuf) == ERROR)
	    message(NO_STATUS, path);
	else if (S_ISDIR(stbuf.st_mode))
	    message(ISDIR, path);
	else {
	    ask(LINKNAME, path, name, 15);
	    if (*name == '\0')
		continue;
	    if (*name == '/' OR link(path, name) == ERROR)
		message(CANT_LINK, name);
	    else {
		insert(FILES, cwd, name);
		delete(POOL, filep);
		}
	    }
	}
    pr_port(FILES);
    pr_port(POOL);
}

/*** Skapa ett bibliotek ***/
do_mkdir()
{
    char name[15];

    ask(DIRNAME, NULL, name, 15);
    if (*name == '\0')
	return;
    if (*name == '/' OR exec("/bin/mkdir", name) == ERROR)
	message(NO_MKDIR, name);
    else {
	insert(DIRS, cwd, name);
	pr_port(DIRS);
	}
}

/*** [ndra filprivilegier ***/
do_chmod()
{
    message(HIT_CE, NULL);
}

/*** G|r en tar c ***/
do_tarc()
{
    message(HIT_CE, NULL);
}

/*** G|r en tar x ***/
do_tarx()
{
    message(HIT_CE, NULL);
}

cd(pt, nr)
register int pt, nr;
{
    char *strrchr();
    register struct file_info *filep;
    register char *p;
    register int i;

    if (pt == DIRS) {
	i = 0;
	for (filep = port[pt].first; filep AND i < nr; filep = filep->next)
	    i++;
	if (filep)
	    if (strcmp(filep->name, ".")
		    AND (strcmp(filep->name, "..") OR strcmp(cwd, "/")))
		/* We aren't trying a cd to "." and we are not standing
		   in "/" and trying a cd to ".." */
		if (chdir(filep->name) == ERROR)
		    message(CANT_CD, filep->name);
		else {
		    if (strcmp(filep->name, "..") == 0) {
			/* Cd to ".." */
			/* Cut off the trailing '/' */
			cwd[strlen(cwd) - 1] = '\0';
			p = strrchr(cwd, '/') + 1;
			*p = '\0';
			}
		    else {
			strcat(cwd, filep->name);
			strcat(cwd, "/");
			}
		    readdir();
		    pr_port(FILES);
		    pr_port(DIRS);
		    }
	}
}
