/*************************************************************************
    fsadm - File system administrator
    1.00 85-09-16 Henrik Norbeck, Luxor Datorer AB
   
    init.c inneh}ller initieringsrutiner samt rutiner f|r signaler och
		terminalkontroll.
*************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/termio.h>
#include <signal.h>
#include "fsadm.h"
#include "struct.h"

/* Meddelanden */
extern int n_lang;
extern char *messages[];
extern char *languages[];

extern int lang;

extern struct viewport port[];
extern int port_h;

extern int ncommands, nfunctions;

extern short cbox_x, cbox_y, fbox_x, fbox_y;

extern char *backup;

/* Terminalparametrar: */
extern int term;	/* Terminaltyp */
extern int term_w;      /* Terminalbredd */
extern int term_h;      /* Terminalh|jd */
extern char *cm;        /* Flytta mark|r */
extern char *clr;       /* T|m sk{rmen */
extern char *so;        /* Standout */
extern char *se;        /* Slut p} standout */
extern char *ku;        /* S{nds av upp}tpilen p} terminalen */
extern char *kd;        /* S{nds av ned}tpilen p} terminalen */
extern char *kl;	/* S{nds av v{nsterpilen p} terminalen */
extern char *kr;	/* S{nds av h|gerpilen p} terminalen */
extern int fontw;	/* Fontens storlek */
extern int fonth;
extern char fontchar;	/* Nuvarande font */

/*** H{mta in terminalparametrar och spr}k ***/
init()
{
   char *malloc();
   char *getenv();
   register char *termbp;
   static char *termarea;
   register char *envlang;
   register int i;

    /* S{tt om terminalmoden */
    sterm(STDIN, RAW);
    /* S{tt om interruptsignalen till goodbye() */
    interrupt(ON);
    /* H{mta terminalparametrar */
    termbp = malloc(1024);
    if ((termarea=getenv("TERM"))==NULL)
        error(NO_TERM);
    term=(strcmp(termarea,"abc1600w")==0 ||
         strcmp(termarea,"win")==0);
    if (tgetent(termbp,termarea)!=1)
        error(NO_ENTRY);
    termarea = malloc(128);
    tgetpar("cm",&cm,&termarea);
    tgetpar("cl",&clr,&termarea);
    tgetpar("so",&so,&termarea);
    tgetpar("se",&se,&termarea);
    if (!tgetpar("ku",&ku,&termarea))
        ku=UP;
    if (!tgetpar("kd",&kd,&termarea))
        kd=DOWN;
    if (!tgetpar("kl",&kl,&termarea))
        kl=LEFT;
    if (!tgetpar("kr",&kr,&termarea))
        kr=RIGHT;
    term_w=tgetnum("co");
    term_h=tgetnum("li");
    free(termbp);
    calcports();
    if (term==WINDOW)
	fontsize();
    /* H{mta spr}k */
    if ((envlang=getenv("LANGUAGE"))!=NULL)
        for (i=0; i<n_lang; i++)
            if (strcmp(envlang, languages[i])==0)
                lang=i;
    /* H{mta namnet p} backupenheten */
    backup=getenv("BACKUP");
}

/*** H{mta en terminalparameter ***/
tgetpar(par,ptr,area)
register char *par, **ptr, **area;
{
   char *tgetstr();

   if ((*ptr=tgetstr(par,area))!=NULL) {
      for (; **ptr>='0' && **ptr<='9'; (*ptr)++);
      return (1);
      }
   else {
      *(*ptr=(*area++))='\0';
      return (0);
      }
}

/*** R{kna ut parametrar till gluggar o dyl ***/
calcports()
{
    register int col_space; /* avst}nd mellan portar och klickboxar */

    if (term_w < MIN_COLS || term_h < MIN_ROWS)
	error(TOO_SMALL);
    col_space = (term_w - MIN_COLS) / 4 + 1;
    if (term == WINDOW) {
	port[FILES].x = 1;
	port[DIRS].x = 4 + PORT_W; /* 1 + PORT_W + 2 + 1 */
	port[POOL].x = 9 + BOXW + 2 * PORT_W + 2 * col_space;
		/* 1 + PORT_W + 2 + 1 + PORT_W + (col_space + 1) + 2 +
		   + BOXW + (col_space + 1) + 1 */
	port[FILES].y = port[DIRS].y = port[POOL].y = term_h - 2;
	port_h = term_h - 4;
	fbox_x = 7 + 2 * PORT_W + col_space;
		/* 1 + PORT_W + 2 + 1 + PORT_W + (col_space + 1) + 2 */
	fbox_y = 7 + (term_h - 7 - nfunctions * (BOXH + BOXD)) / 2;
	cbox_x = 12 + BOXW + 3 * PORT_W + 3 * col_space;
		/* 1 + PORT_W + 2 + 1 + PORT_W + (col_space + 1) + 2 + BOXW +
		   + (col_space + 1) + 1 + PORT_W + (col_space + 1) + 2 */
	cbox_y = 2 + ncommands * (BOXH + BOXD);
	}
    else {
	port[FILES].x = 2;
	port[DIRS].x = 6 + PORT_W; /* 2 + PORT_W + 1 + 3 */
	port[POOL].x = 12 + BOXW + 2 * PORT_W + 2 * col_space;
		     /* 2 + PORT_W + 1 + 3 + PORT_W + col_space + 3 + BOXW +
			+ col_space + 3 */
	port[FILES].y = port[DIRS].y = port[POOL].y = term_h - 3;
	port_h = term_h - 6;
	fbox_x = 9 + 2 * PORT_W + col_space;
		/* 2 + PORT_W + 1 + 3 + PORT_W + col_space + 3 */
	fbox_y = 7 + (term_h - 7 - nfunctions * (BOXH + BOXD)) / 2;
	cbox_x = 15 + BOXW + 3 * PORT_W + 3 * col_space;
		/* 2 + PORT_W + 1 + 3 + PORT_W + col_space + 3 + BOXW +
		   + col_space + 3 + PORT_W + col_space + 3 */
	cbox_y = 2 + ncommands * (BOXH + BOXD);
	}
}

/*** H{mta fontstorleken f|r f|nstret ***/
fontsize()
{
    register int i;
    register char c;

    printf("\033:3n");
    for (i = 0; i < 2; i += (getchar() == ';'))
	;
    fontw = 0;
    while ((c = getchar()) != ';')
	fontw = fontw * 10 + c - '0';
    fonth = 0;
    while ((c = getchar()) != ';')
	fonth = fonth * 10 + c - '0';
    while (getchar() != ';')
	;
    fontchar = getchar();
    while (getchar() != 'W');
}

/*** S{tt eller }terst{ll terminalmoden ***/
sterm(fd, option)
int fd, option;
{
   struct termio tty;
   static struct termio oldtty;

   if (option == RAW) {
      ioctl(fd, TCGETA, &tty);
      oldtty = tty;
      tty.c_lflag &= ~(ICANON | ECHO);
      tty.c_iflag &= ~IXON;
      ioctl(fd, TCSETA, &tty);
      }
   if (option == OLD) 
      ioctl(fd, TCSETA, &oldtty);
}

/*** St{ng av / s{tt p} omritningen ***/
refresh(f)
short f;
{
    int draw();

    signal(SIGUSR1, f ? draw : SIG_IGN);
}

/*** S{tt hur programmet kan avbrytas ***/
interrupt(f)
short f;
{
    int goodbye();

    signal(SIGINT, f ? goodbye : SIG_IGN);
}
