/*************************************************************************
    fsadm - File system administrator
    1.00 85-09-16 Henrik Norbeck, Luxor Datorer AB
   
    port.c inneh}ller rutiner f|r porthantering
*************************************************************************/

#include <stdio.h>
#include "fsadm.h"
#include "struct.h"

/* Meddelanden och menyval */
extern int n_lang;
extern int lang;

/* Terminalparametrar */
extern int term;
extern char *cm;

extern char cwd[];

extern struct viewport port[];
extern int port_h;
extern int p_port_h, p_port_w;

/*** T|m inneh}llet i en viewport ***/
reset(pt)
register short pt;
{
    register struct file_info *filep, *next;

    for (filep = port[pt].filelist; filep; filep = next) {
	free(filep->path);
	next = filep->next;
	free(filep);
	}
    port[pt].first = port[pt].filelist = NULL;
    port[pt].cnt = 0;
}

/*** S{tt in ett filnamn i en viewport ***/
insert(pt, path, name)
register int pt;
register char *path, *name;
{
    char *alloc(), *strcpy(), *strcat();
    struct file_info *search();
    register int plen, nlen;
    register struct file_info *filep, *new;

    filep = search(name, pt);
    new = (struct file_info *) alloc(sizeof(struct file_info));
    if (filep) {
	new->next = filep->next;
	filep->next = new;
	}
    else {
	new->next = port[pt].filelist;
	port[pt].filelist = new;
	}
    if (port[pt].first == new->next OR port[pt].first == NULL)
	port[pt].first = new;
    port[pt].cnt++;
    plen = strlen(path);
    nlen = strlen(name);
    new->path = alloc(plen+nlen+1);
    new->name = new->path + plen;
    strcpy(new->path, path);
    strcat(new->path, name);
}

/*** Leta efter filnamn i en viewport ***/
struct file_info *search(name, pt)
register char *name;
register int pt;
{
    register struct file_info *filep, *prev;

    prev = NULL;
    for (filep = port[pt].filelist; filep; filep = filep->next) {
	if (strcmp(name, filep->name) < 0)
	    break;
	prev = filep;
	}
    return(prev);
}

/*** Skriv ut inneh}llet i en port ***/
pr_port(pt)
register int pt;
{
    register struct file_info *filep;
    register int i, j, x = port[pt].x, y = port[pt].y;

    /* Skriv filnamnen */
    i = 1;
    for (filep = port[pt].first; filep && i <= port_h; filep = filep->next) {
	printf("%s%-14s", tgoto(cm, x, y - port_h + i), 
			filep->name);
	i++;
	}
    /* Rensa nederdelen av gluggen */
    for (; i <= port_h; i++)
	printf("%s              ",
		tgoto(cm, x, y - port_h + i));
    /* Visa hur mycket som finns ovanf|r resp. under gluggen */
    if (term == WINDOW) {
	register int ph = p_port_h, pw = p_port_w,
		px = port[pt].px, py = port[pt].py,
		above, below;

	/* R{kna antalet filer nedanf|r gluggen */
	for (below = 0; filep; filep = filep->next)
	    below++;
	/* R{kna antalet filer ovanf|r gluggen */
	filep = port[pt].filelist;
	for (above = 0; filep != port[pt].first; filep = filep->next)
	    above++;
	if (port[pt].cnt) {
	    above = (ph - 50) * above / port[pt].cnt;
	    below = (ph - 50) * below / port[pt].cnt;
	    }
	else
	    above = below = 0;
	printf("\033:%d;%dm", px+pw+3, py+25);		/* move */
	printf("\033:%d;%df", px+pw+20, py+ph-27);	/* fill white box */

	if (above > 2) {
	    printf("\033:%d;%dm", px+pw+3, py+ph-26-above);	/* move */
	    printf("\033:%d;%d;;1d", px+pw+20, py+ph-26-above);	/* line */
	    printf("\033:%d;%dm", px+pw+3, py+ph-25-above);	/* move */
	    printf("\033:%d;%d;8;1f", px+pw+20, py+ph-27);	/* fill box */
	    }
	if (below > 2) {
	    printf("\033:%d;%dm", px+pw+3, py+24+below);	/* move */
	    printf("\033:%d;%d;;1d", px+pw+20, py+24+below);	/* line */
	    printf("\033:%d;%dm", px+pw+3, py+23+below);	/* move */
	    printf("\033:%d;%d;8;1f", px+pw+20, py+25);		/* fill box */
	    }
	}
    else {
	if (port[pt].first != port[pt].filelist)
	    printf("%s+", tgoto(cm, x + 2, y - port_h));
	if (filep != NULL)
	    printf("%s+", tgoto(cm, x + 2, y + 1));
	}
}

/*** Tag bort en fil ur en port ***/
delete(pt, filep)
register int pt;
register struct file_info *filep;
{
    register struct file_info *prevp;

    /* L{nka ur filen */
    port[pt].cnt--;
    if (port[pt].filelist == filep)
	port[pt].filelist = port[pt].first = filep->next;
    else {
	/* Hitta f|reg}ende fil */
	for (prevp = port[pt].filelist; prevp->next != filep;
		prevp = prevp->next)
	    ;
	prevp->next = filep->next;
	if (port[pt].first == filep)
	    port[pt].first = prevp;
	}
    /* Tag bort den */
    free(filep->path);
    free(filep);
}

/*** Tag bort en fil ur poolen, och ur pt om den ligger
	i nuvarande biblioteket ***/
del_rm(filep, pt)
register struct file_info *filep;
register int pt;
{
    char *getpath();
    struct file_info *search();
    register struct file_info *filep2;
    register int flag = FALSE;

    if (*getpath(filep) != '/')
	if (filep2 = search(filep->name, pt)) {
	    delete(pt, filep2);
	    flag = TRUE;
	    }
    delete(POOL, filep);
    return (flag);
}

/*** \verf|r en fil eller ett bibliotek till poolen ***/
mark(pt, nr)
register int pt, nr;
{
    register struct file_info *filep, *prevp;
    register int i;

    if (nr == ALL)
	if (pt == POOL) {
	    reset(POOL);
	    pr_port(POOL);
	    }
	else {
	    for (filep = port[pt].filelist; filep; filep = filep->next) {
		/* Kolla om filen redan fanns i poolen */
		for (prevp = port[POOL].filelist; prevp; prevp = prevp->next)
		    if (strcmp(filep->path, prevp->path) == 0)
			break;
		if (prevp == NULL)
		    /* L{gg in filen i poolen */
		    insert(POOL, cwd, filep->name);
		}
	    pr_port(POOL);
	    }
    else {
	i = 0;
	for (filep = port[pt].first; filep AND i < nr; filep = filep->next)
	    i++;
	if (filep)
	    if (pt == POOL) {
		delete(POOL, filep);
		pr_port(POOL);
		}
	    else {
		/* Kolla om filen redan fanns i poolen */
		for (prevp = port[POOL].filelist; prevp; prevp = prevp->next)
		    if (strcmp(filep->path, prevp->path) == 0)
			break;
		if (prevp == NULL) {
		    /* L{gg in filen i poolen */
		    insert(POOL, cwd, filep->name);
		    pr_port(POOL);
		    }
		}
	}
}

/*** Rulla en glugg ***/
scroll(pt, nr)
{
    register struct file_info *filep;
    register int cnt;

    if (nr) {
	/* Nedre pilen => rulla upp}t */
	/* Kolla f|rst om det finns n}gra filer nedanf|r gluggen */
	filep = port[pt].first;
	for (cnt = 0; cnt < port_h AND filep; cnt++)
	    filep = filep->next;
	for (cnt = 0; cnt < port_h/2 AND filep; cnt++) {
	    port[pt].first = port[pt].first->next;
	    filep = filep->next;
	    }
	if (cnt)
	    pr_port(pt);
	}
    else {
	/* \vre pilen => rulla ned}t */
	for (cnt = 0; cnt < port_h/2 AND port[pt].first != port[pt].filelist;
			cnt++) {
	    filep = port[pt].filelist;
	    while (filep->next != port[pt].first)
		filep = filep->next;
	    port[pt].first = filep;
	    }
	if (cnt)
	    pr_port(pt);
	}
}

/*** G|r malloc() och skrik om det ej finns minne kvar ***/
char *alloc(n)
{
    char *p, *malloc();

    if ((p = malloc(n)) == NULL)
	error(NO_MEMORY);
    return(p);
}
