/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: first version
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11-28 ldab/hn: It's a shell now
	by Henrik Norbeck, Luxor Datorer AB 

	do_cmd.c Contains functions to execute commands

	do_cmd	Executes a command line.
	search	Searches for a delimiter in a command string
	exec	Executes a command outside msh
***********************************************************/

/* Header files */
#include <stdio.h>
#include <signal.h>
#include "msh.h"

extern int (*sigquit)(), (*sigint)();

extern char *homedir;
/* Terminal parameters */
extern char *clr;

/***	execute user choice

	reset terminal capability and
	execute command string, monitoring
	error status.
***/
do_cmd(cmd_str)
char *cmd_str;
{
    char *noescquot(), *search(), *strrchr();
    register short int ret_flag, argc, i;
    register char *oldpos, *argv[32], c, *name, *cmd;
    char cmd_buf[1024];
    char argbuf[1024], *argp;
    short in_cmd, in_str=TRUE, assign;
    int mode;

    strcpy(cmd_buf, cmd_str);
    cmd = cmd_buf;
    /* Scan command string */
    while (*cmd && in_str) {
	argc = 0;
	ret_flag = assign = FALSE;
	in_cmd = TRUE;
	while (in_cmd) {	/* Scan the command */
	    oldpos = cmd;
	    c = *(cmd=search(cmd));
	    *cmd++ = '\0';	/* Terminate this component of the */
				/* command string with a null char */
	    switch (c) {
		case '\0':
		    in_str = FALSE;
		case ';':	/* End of command */
		    in_cmd = FALSE;
		case ' ':	/* End of component */
		    if (c)
			while(*cmd == ' ')
			    cmd++;
		    if (*cmd==';' || NOT (in_str=(*cmd!='\0')))
			in_cmd = FALSE;
		    argv[argc++] = noescquot(oldpos);
		    break;
		case '=':	/* Environment variable assignment */
		    assign = TRUE;
		    argv[0] = cmd;
		    while ((in_str = *(cmd=search(cmd))) && *cmd!=';')
			cmd++;
		    *cmd++ = '\0';
		    if (argc) {
			pr_msg(BAD_CMD);
			ret_flag = TRUE;
			break;
			}
		    noescquot(oldpos);
		    noescquot(argv[0]);
		    setvar(oldpos, argv[0]);
		    setenv();
		    in_cmd = FALSE;
		    break;
		}
	    }
	if (argc==0 || assign || *argv[0]=='#')
	    continue;

	sterm(OFF);	/* Reset and clear terminal */
	if (clr) {
	    print(clr);
	    crtbrk();
	    }

	argv[argc] = NULL;	/* End argument vector with a NULL pointer */

	argp = argbuf;
	for (i = 0; i < argc; i++)	/* Scan '?' arguments */
	    if (*argv[i] == '?') {
		printf("%s: ", argv[i]+1);
		argv[i] = argp;
		while ((*argp++ = getchar()) != '\n');
		*(argp-1) = '\0';
		}

	name = argv[0];
	if ((argv[0]=strrchr(name, '/')) == NULL)
	    argv[0] = name;	/* Extract argv[0] from command name */

	if (*name == '!')	/* Exit from msh */
	    exit(0);
	if (*name == '*') {	/* Wait for a return after command */
	    name++;
	    ret_flag++;
	    }
	if (strcmp(name, "cd")==0) {	/* Change directory */
	    if (argv[1]==NULL)
		argv[1] = homedir;
	    if (chdir(argv[1])==ERROR) {
		pr_msg(BAD_CMD);
		ret_flag = TRUE;
		}
	    }
	else if (strcmp(name, "exit")==0)/* Exit from msh */
	    exit(0);
	else if (strcmp(name, "umask")==0)
	    if (argv[1]!=NULL) {	/* Set umask */
		if (sscanf(argv[1], "%o", &mode)==1)
		    umask(mode);
		}
	    else			/* Get umask */
		printf("%o\n", umask(mode = umask(0)));
	else
	    if (exec(name, argv/*, wait*/) == ERROR)
		ret_flag = TRUE;
wait_ret:
	if (ret_flag) {
	    pr_msg(0);
	    while(getchar()!='\n');
	    }
	sterm(ON);
	}
}

/***	Search for a delimiter etc in a command string ***/
char *search(str, match)
register char *str;
{
    register char c, *tmp;

    while (c = *str) {
	if (delimiter(c))
	    break;
	str++;
	if (c == '\\')	/* Skipped escaped characters */
	    if (NOT *str)
		break;
	    else
		str++;
	else if (c=='"' || c=='\'') {	/* Skip strings delimited by " or ' */
	    for (; *str && *str!=c; str++)
		if (*str == '\\')	/* Skip escaped character */
		    if (NOT *(++str))
			break;
	    str++;
	    }
	}
    return(str);
}

/***	Check if a character is a delimiter ***/
delimiter(c)
char c;
{
    char *strchr();

    return (strchr(" ;=", c) != NULL);
}

/***	Execute a command ***/
exec(name, argv/*, wait_flag*/)
register char *name, *argv[];
/*register short wait_flag;*/
{
    int f_nr, status;

    if ((f_nr=fork()) == ERROR) {
	pr_msg(NO_PS);
	return(ERROR);
	}
    else if (f_nr == 0) {
	signal(SIGQUIT, SIG_DFL);
	signal(SIGINT, SIG_DFL);
	execvp(name, argv);
	exit(0);
	}
    else
/*	if (wait_flag) {*/
	    while(wait(&status) != f_nr);
	    return(0);
/*	    }
	else {
	    pr_msg(PS_NR);
	    printf("%d\n", f_nr);
	    return(0);
/*	    }*/
}
