/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: first version.
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11-28 ldab/hn: It's a shell now
	by Henrik Norbeck, Luxor Datorer AB 

	init.c Contains initialization functions:

	terminit - reads terminal data from /etc/termcap
	getlang	 - gets the language
	sterm	 - sets the terminal in special mode
	getsysid - gets the system id from /etc/systemid
	cgrpid   - gets the group name from /etc/group
***********************************************************/

/* Header files */
#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/termio.h>
#include <grp.h>
#include "msh.h"

/* language stuff */
extern int n_lang;
extern char *languages[];
int lang=0;

/* user information storage */
extern char systemid[];	/* system identification */

/*	The following are string pointers used by the
	terminal handlers. the pointers	use a static
	area (termarea, see terminit())	to store the
	string values. */
char *cur=NULL;		/* cursor placement */
char *standout=NULL;	/* standout (inverse) */
char *standoff=NULL;	/* no standout (normal) */
char *clr=NULL;		/* clear screen */
char *cur_off=NULL;	/* Cursor off */
char *cur_on=NULL;	/* Cursor on */
char *ku=NULL;		/* up arrow */
char *kd=NULL;		/* down arrow */
/* terminal size parameters */
int  	width;			/* screen width (columns) */
int  	length;			/* screen length (rows) */


/*** terminit() does terminal initialization */
terminit()
{
    char *malloc();
    char *getvar();
    register char *termbp;			/* temp buffer     */
    static char *termarea;			/* term char area  */

    termbp = malloc(1024);
    if ((termarea = getvar("TERM")) == NULL)
	panic("TERM environment parameter not found");
    if (tgetent(termbp, termarea) != 1)	/* get entry */
	panic("no terminal parameters");
    termarea = malloc(128);
    tgetpar("cm", &cur, &termarea);
    tgetpar("so", &standout, &termarea);
    tgetpar("se", &standoff, &termarea);
    tgetpar("cl", &clr, &termarea);
    tgetpar("CF", &cur_off, &termarea);
    tgetpar("CO", &cur_on, &termarea);
    printf("%s", cur_off);
    tgetpar("ku", &ku, &termarea);
    tgetpar("kd", &kd, &termarea);
    width = tgetnum("co");
    length = tgetnum("li");
    free(termbp);
}

tgetpar(par, ptr, area)
register char *par, **ptr, **area;
{
    char *tgetstr();

    if ((*ptr = tgetstr(par, area)) != NULL) {
	for (;**ptr>='0' && **ptr<='9';(*ptr)++);
	return (TRUE);
	}
    else {
	*(*ptr = (*area)++) = '\0';
	return (FALSE);
	}
}

/***    Get language number to be used ***/
getlang()
{
    int i;
    char *langbuf, *getvar();

    if ((langbuf = getvar("LANGUAGE")) != NULL)
        for (i=0; i<n_lang; i++)
            if (strcmp(langbuf, languages[i]) == 0) {
                lang = i;
                break;
                }
}

/***	sterm(ON)	set special mode, and turn cursor off
	sterm(OFF)	reset to normal mode, and turn cursor on again
*/
sterm(n) int n;
{
    struct termio tty;

    ioctl(STDIN, TCGETA, &tty); 

    /* immediate action	lflag:ICANON
       no echo		lflag:ECHO
       no signals	lflag:ISIG

       no stop/start	iflag:IXON
    */

    if (n == ON) {
	ResBit(tty.c_lflag, ICANON | ECHO);
	ResBit(tty.c_iflag, IXON);
	}
    else {
	SetBit(tty.c_lflag, ICANON | ECHO);
	SetBit(tty.c_iflag, IXON);
	}
    ioctl(STDIN, TCSETA, &tty);
    if (n == ON && cur_off)
	printf("%s", cur_off);
    if (n == OFF && cur_on)
	printf("%s", cur_on);
}

/*** get system identity from the file /etc/systemid */
getsysid()
{
	FILE *fp, *fopen();
	if ((fp = fopen("/etc/systemid", "r")) != NULL )
	{
		fscanf(fp, "%s", systemid);
		fclose(fp);
	}
	else
		*systemid = '\0';
}

/***	Get group name from /etc/group ***/
char *cgrpid(s)
char *s;
{
    struct group *getgrgid(), *grpbuf, gbuf2;
    static char c='\0';

    grpbuf = getgrgid(getgid());
    if (grpbuf == NULL) {
	grpbuf = &gbuf2;
	grpbuf->gr_name = &c;
	}
    if (s) {
	strcpy(s, grpbuf->gr_name);
	return(NULL);
	}
    else
	return(grpbuf->gr_name);
}
