/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: First version
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11- ldab/hn: It's a shell now
	by Henrik Norbeck, Luxor Datorer AB 

	Contents of the files:
	main.c		The main program
	init.c		Initialization
	output.c 	Output functions
	var.c		Environment variable handling
	menuread.c	Menu initialization
	menu.c		Menu handling
	language.c	Language dependent stuff
	msh.h		Header file

	main.c Contains these functions:

	main		The main program
	do_profile	Executes .profile
	getcwd		Get current working directory
			(missing in standard c library)
	mkfilename	Make a path name out of a directory name
			and a file name.
***********************************************************/

/* Header files */
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <signal.h>
#include "msh.h"

/* user information storage */
char *homedir;		/* home directory */
char userid[L_cuserid];	/* user identification */
char grpid[L_cuserid];
char systemid[40];	/* system identification */
char *g_program;	/* global prog name */
char structfile[128];

main(argc,argv)
int argc;
char **argv;
{
    char *getvar(), *cuserid(), *cgrpid(), *strrchr(), *mkfilename();
    FILE *profile;

    g_program = argv[0];
    if (*g_program == '-')
	g_program++;
    if (NOT isatty(STDIN))
	panic("Not started from a terminal");
    varinit();
    signal(SIGQUIT, SIG_IGN);
    signal(SIGINT, SIG_IGN);
    if (argc && *argv[0]=='-' && (profile=fopen(".profile", "r"))!=NULL)
	do_profile(profile);
    if ((homedir = getvar("HOME")) == NULL)
	homedir = "";
    terminit();	/* output pars */
    sterm(ON);	/* unique mode */
    if (argc > 2)
	panic("Usage: msh [structfile]");
    if (argc == 2)
	if (*argv[1] == '/')
	    strcpy(structfile, argv[1]);
	else {
	    getcwd(structfile, 128);
	    mkfilename(structfile, argv[1]);
	    }
    else {
        strcpy(structfile, homedir);
        strcat(structfile, MENUFILE);
        }
    if (access(structfile, 04) == ERROR) {	/* READ */
        strcpy(structfile, GRPDIR);
        strcat(structfile, grpid);
        strcat(structfile, MENUFILE);
        if (argc==2 || access(structfile, 04) == ERROR) {
            strcat(strcpy(structfile, OTHDIR), MENUFILE);
            if (access(structfile, 04) == ERROR)
                panic("no structure file");
            }
        }

    strcpy(userid, cuserid((char) 0));
    strcpy(grpid, cgrpid((char) 0));
    getsysid();

    menucall();
	
    exit(0);
}

/***	Execute commands from .profile ***/
do_profile(profile)
FILE *profile;
{
    char *getline(), cmdbuf[256];

    while (getline(profile, cmdbuf) != NULL)
	do_cmd(cmdbuf);
}

/***	Get current working directory ***/
char name[128], path[128];

getcwd( d_name, maxlen )
char *d_name;
int maxlen;
{
    int fd;
    struct direct dirbuf;

    fd=open( ".", O_RDONLY );
    read( fd, (char *) &dirbuf, sizeof( dirbuf ) );
    close( fd );
    *name = '\0';
    strcpy( path, ".." );
    scanpath( dirbuf.d_ino );
    strcpy( d_name, name );
    if ( d_name[0]=='\0' )
        strcpy( d_name, "/" );
    return ( strlen( d_name ) );
}

scanpath( pi )
ino_t pi;
{
    int fd;
    struct direct dirbuf;
    ino_t ti;

    fd=open( path, O_RDONLY );
    read( fd, (char *) &dirbuf, sizeof( dirbuf) );
    if ( ( ti = dirbuf.d_ino ) == pi ) return;
    do {
        read( fd, (char *) &dirbuf, sizeof( dirbuf ) );
        } while ( pi != dirbuf.d_ino );
    close( fd );
    strcat( path, "/.." );
    scanpath( ti );
    strcat( strcat( name, "/" ), dirbuf.d_name );
}

/***	Make a path name out of a directory name and a file name ***/
char *mkfilename(dir, file)
char *dir, *file;
{
    char *slash;

    while (strncmp(file, "../", 3)==0 || strncmp(file, "./", 2)==0) {
	if (file[1] == '/')
	    file += 2;
	else {
	    if ((slash=strrchr(dir, '/')) > dir)
		*slash = '\0';
	    file += 3;
	    }
	}
    strcat(dir, "/");
    strcat(dir, file);
    return (dir);
}
