/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: first version
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11- ldab/hn: It's a shell now
	by Henrik Norbeck, Luxor Datorer AB 

	menu.c Contains menu handling
	menucall   - The main menu loop.
	menuselect - Select a choice from a menu.
	menuscreen - Prints the menu screen.
	lineupdate - Marks the current menu line
	menuline   - Prints a menu line
***********************************************************/

/* Header files */
#include <stdio.h>
#include "msh.h"

/* user information storage */
extern char *homedir;   /* home directory */
extern char userid[];   /* user identification */
extern char systemid[]; /* system identification */
extern char *g_program; /* global prog name */

/*	The following are string pointers used by the
	terminal handlers. the pointers	use a static
	area (termarea, see terminit())	to store the
	string values. ***/
extern char *standout;		/* standout (inverse) */
extern char *standoff;		/* no standout (normal) */
extern char *clr;		/* clear screen */
extern char *ku;		/* up arrow */
extern char *kd;		/* down arrow */
/* terminal size parameters */
extern int width;		/* screen width (columns) */
extern int length;		/* screen length (rows) */

/*	menu choice registers */
int 	current = 0;		/* is here */
int	oldline = (-1); 	/* was here */
int	homepos;		/* starts here */

/*	string arrays for the menu */
char mbuf[MENUSIZE][MENULINE];	/* Menu texts */
char pbuf[MENUSIZE][PROGLINE];	/* Program names */
char nbuf[MENUSIZE][NAMELINE];  /* Short names */

char menustack[STACKSIZE][NAMELINE];
short stackpointer = 0; /* Points to next empty entry */

char menu_name[PROGLINE];
int menusiz;
/*** Menucall() is the menu loop */
menucall()
{
    char cwd[40], c;
    int choice, i, n;

    *menu_name = '\0';
    menusiz = getmenu();
    strcpy(menu_name, nbuf[0]);
    homepos = length - menusiz - 4;

    do {
	menuscreen(menusiz);
	choice = menuselect(menusiz);
	if (*pbuf[choice] == '$') {
	    switch (pbuf[choice][1]) {
		case '!':
		    for (i=1; pbuf[choice][i]=='!'; i++)
			if (stackpointer > 0)
			    strcpy(menu_name, menustack[--stackpointer]);
			else
			    do_cmd("!");
		    break;
		case '&':
		    strcpy(menu_name+1, pbuf[choice]+2);
		    *menu_name = '$';
		    break;
		default:
		    i=0;
		    while (pbuf[choice][i]!='\0') {
			n = i++;
			while (pbuf[choice][i]!='$' && pbuf[choice][i]!='\0')
			    i++;
			if (stackpointer < STACKSIZE) {
			    strcpy(menustack[stackpointer++], menu_name);
			    strncpy(menu_name, pbuf[choice]+n, i-n);
			    menu_name[i-n]='\0';
			    }
			else
			    panic("menu stack overflow");
			}
		}
	    menusiz = getmenu();		/* should be             */
						/* getmenu(pbuf[choice]) */
	    homepos = length - menusiz - 4;
	    current = 0;
	    oldline = (-1);

	    if (*pbuf[0] != '\0')
		do_cmd(pbuf[0]);	/* Menu start command */
	    }
	else
	    do_cmd(pbuf[choice]);
    } while (1);
}

/***	select in menu

	this function handles the current pointer
*/
menuselect(menusiz) short int menusiz;
{
    register char c;
    register short int terminate, up_flag, down_flag, i;
    register int menulen = menusiz - 1;

    while (1) {
	lineupdate(menusiz);
	printat(homepos , 0 , "");
	crtbrk();
	terminate = 0;
	up_flag = down_flag = TRUE;
	for(i=0; up_flag || down_flag; i++) {
	    c = getchar();
	    if (up_flag)	/* up arrow */
		if (c != ku[i])
		    up_flag = FALSE;
		else if (ku[i+1] == '\0') {
		    if (current > 1)
			current--;
		    break;
		    }
	    if (down_flag)	/* down arrow */
		if (c != kd[i])
		    down_flag = FALSE;
		else if (kd[i+1] == '\0') {
		    if (current < menulen)
			current++;
		    break;
		    }
	    if ( !(down_flag || up_flag) ) {
		if (c == CTRL_D) {	/* Exit from this menu */
		    terminate++;
		    current = menulen;
		    }
		else if (c == CTRL_L)	/* Redraw screen */
		    menuscreen(menusiz);
/*		else if (c == CTRL_P && current > 1)
/*		    current--;		/* Up one row */
/*		else if (c == CTRL_N && current < menulen)
/*		    current++;		/* Down one row */
		else if (c == '\n')
		    terminate++;
		else if (c == '0')
		    current = menulen;
		else if (c > '0') { 	/* Menu choice */
		    if (c > '`')
			c -= 32;	/* upper case */
		    if (c <= '9')
			c -= '0';	/* number */
		    else if (c > '@')
			c -= ('A'-10);	/* letter */
		    if ( c > 0 && c < menulen )
			current = c;
	    	    }
		break;
		}
	    }
	if (terminate && current)
	    return(current);
	}
}

/***	print menu screen */
menuscreen(menusiz) int menusiz;
{
    register short int n;

    write(STDOUT, "   ", 3); /* Wake ABCute */

    print(clr);
    printat(length - 2 , 0 , standout);
    print(" ABCesam ver 1.00 hos ");
    print(userid);
    if (*systemid) {
	print(" p} ");
	print(systemid);
	}
    for (n = 10; n-- >= 0;)
	print(" ");
    print(standoff);
	
    for (n = 0 ; n < menusiz ; n++)
	menuline(NOMARK, n, menusiz);
}

/***	update menu line

	when a entry is selected, the standout box
	is moved to the new position, indicated by
	'current'. this is done by rewriting the old
	position with normal mode, and then writing
	the new position. if the standout mode is not
	available, the standout is simulated by an
	asterisk typed as the first character on the
	current menu line.
*/
lineupdate(menusiz)
register short int menusiz;
{
    if (oldline != current && oldline != -1)
	menuline( NOMARK, oldline, menusiz); /* retype old */
    print( standout );
    menuline( MARK, current, menusiz );
    print( standoff );
    oldline = current;	/* remember pos */
}

/***	print menu line */
menuline(str, a, menusiz)
char *str;
register short int a;
short int menusiz; 
{
    register short int n = a;
    char c = 0;

    if (a) {
	if (a == menusiz - 1)
	    c = '0';
	else
	    c = (char) n < 10 ? n + '0' : n + ('a' - 10); 
	    n++;
	}
    printat( homepos + n , 0 , str );
    if (c) {
	print(&c);
	print(". ");
	}
    print(mbuf[a]);
    print( " " );
}
