/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: first version
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11- ldab/hn: It's a shell now
	by Henrik Norbeck, Luxor Datorer AB 

	output.c Contains buffer oriented output functions

	printat(r,k,s)	- print string at given position
	cpos(r,k)	- set cursor
	print(s)	- print text
	crtbrk()	- flush buffer contents to screen
	pr_msg()	- prints a message
	panic		- prints an error message and exits
***********************************************************/

/* Header files */
#include <stdio.h>
#include "msh.h"

/***	Language variables ***/
extern int n_lang;
extern int lang;
extern char *message[];

extern char *g_program; /* global prog name */

/***	The following are string pointers used by the
	terminal handlers. the pointers	use a static
	area (termarea, see terminit())	to store the
	string values. ***/
extern char *cur;		/* cursor placement */
extern char *standout;		/* standout (inverse) */
extern char *standoff;		/* no standout (normal) */
extern char *clr;		/* clear screen */
/* terminal size parameters */
extern int width;		/* screen width (columns) */
extern int length;		/* screen length (rows) */

static char crtbuf[BUFOUT];
static char *crtptr = crtbuf;

/* Printat() prints a string at a given position */
printat(r,k,str) int r,k; char *str;
{
	cpos(r,k); print(str);
}

/* Cpos() positions the cursor */
cpos(r,k) int r,k;
{
	char *tgoto();
	print(tgoto(cur,k,r));
}

/* Crtbrk() flushes the output buffer to the terminal */
crtbrk()
{
	write(STDOUT,crtbuf,(unsigned int) (crtptr - crtbuf));
	crtptr = crtbuf;
}

/* Print() prints text in the buffer */
print(str) register char *str;
{
	while (*crtptr++ = *str++);
	crtptr--;
}

/* Print a message */
pr_msg(n)
int n;
{
    printf("%s", message[n_lang*n+lang]);
}

/*** panic() prints an error message, resets the terminal and exits */
panic(s) char *s;
{
    char *tgoto();

    if (cur)
        fprintf(stderr, "%s", tgoto(cur, 0, length-1));
    fprintf(stderr, "%s: %s\n", g_program, s);
    sterm(OFF);
    exit(1);
}
