/***********************************************************
	msh - menu shell

	0.00 85-08-15 ldab/fl: first version.
	1.00 85-10-14 ldab/hn: Can handle more than one menu
	2.00 85-11- ldab/hn: It's a shell now.
	by Henrik Norbeck, Luxor Datorer AB 

	var.c Contains functions for handling environment variables

	varinit	Initiates the variables.
	getvar	Gets a variable.
	setvar	Sets a variable.
	setenv	Sets the environment
***********************************************************/

/* Header files */
#include <stdio.h>
#include "msh.h"

extern char **environ;
struct var varlist, *lastvar;

/***	Initiate environment variables ***/
varinit()
{
    char *value, *getvar();
    int i;

    varlist.value = varlist.name = NULL;
    varlist.next = NULL;
    lastvar = &varlist;

    for (i=0; *environ; environ++) {	/* Get original environment */
	for (value = *environ; *value != '='; value++);
	*value = '\0';
	setvar(*environ, value+1);
	}

    if (getvar("PATH") == NULL)
	setvar("PATH", ":/bin:/usr/bin");
    if (getvar("LANGUAGE") == NULL)
	setvar("LANGUAGE", "english");
    environ = NULL;
    setenv();
}

/***	Get a variable value ***/
char *getvar(name)
char *name;
{
    struct var *varp;

    for (varp=varlist.next; varp; varp = varp->next)
	if (strcmp(varp->name, name) == 0)
	    return (varp->value);
    return (NULL);
}

/***	Set a variable value ***/
setvar(name, value)
char *name, *value;
{
    char *malloc();

    lastvar->next = (struct var *) malloc(sizeof(struct var));
    lastvar = lastvar->next;
    lastvar->name = name;
    lastvar->value = value;
    lastvar->next = NULL;
}

/***	Set the environment ***/
setenv()
{
    char *malloc();
    register int count = 1;
    register char **env;
    register struct var *varp;

    if (environ) {	/* Free old environment */
	env = environ;
	while (*env) {
	    free(*env);
	    env++;
	    }
	free(environ);
	}

    for (varp = varlist.next; varp; varp = varp->next)
	count++;	/* Count number of variables */
    environ = env = (char **) calloc(count, sizeof(char *));

    /* Set new environment */
    for (varp = varlist.next; varp; varp = varp->next) {
	*env = malloc(strlen(varp->name) + strlen(varp->value) + 2);
	strcpy(*env, varp->name);
	strcat(*env, "=");
	strcat(*env, varp->value);
	env++;
	}
    *env = NULL;
}
